@@ -20,7 +20,6 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import contextlib
 import dataiter
 import datetime
 import itertools
@@ -77,7 +76,7 @@ class DataFrameColumn(Vector):
 
     @classmethod
     def _sequencify(cls, object):
-        if isinstance(object, (list, tuple, np.ndarray)):
+        if isinstance(object, (np.ndarray, list, tuple)):
             return object
         if (object is None or
             np.isscalar(object) or
@@ -192,7 +191,7 @@ class DataFrame(dict):
         >>> # with the shorthand helpers, but for complicated calculations,
         >>> # you might need custom lambda functions.
         >>> data.group_by("hood").aggregate(n=di.count(), price=di.mean("price"))
-        >>> data.group_by("hood").aggregate(n=lambda x: x.nrow, price=lambda x: np.nanmean(x.price))
+        >>> data.group_by("hood").aggregate(n=lambda x: x.nrow, price=lambda x: x.price.mean())
         """
         group_colnames = self._group_colnames
         data = self.sort(**dict.fromkeys(group_colnames, 1))
@@ -206,17 +205,25 @@ class DataFrame(dict):
             data._group_ = np.repeat(groups, n)
         slices = None
         for colname, function in colname_function_pairs.items():
-            uses_numba = getattr(function, "numba", False)
-            if uses_numba:
-                # Numba-accelerated aggregation functions raise
-                # NotImplementedError if given an unsupported dtype.
-                with contextlib.suppress(NotImplementedError):
-                    stat[colname] = function(data)
-                    continue
-                function = function.fallback
-            if slices is None:
-                slices = [data._view_rows(x) for x in indices]
-            stat[colname] = [function(x) for x in slices]
+            if getattr(function, "numba", False):
+                # function might leave Nones in its output,
+                # once those are replaced with the proper default
+                # we can do a fast conversion to DataFrameColumn.
+                column = function(data)
+                default = function.default
+                for i in range(len(column)):
+                    if column[i] is None:
+                        column[i] = default
+                assert len(column) == stat.nrow
+                column = DataFrameColumn.fast(column)
+                stat[colname] = column
+            else:
+                # When using an arbitrary function, we cannot know
+                # what special values to expect and thus we end up
+                # needing to use the slow Vector.__init__.
+                if slices is None:
+                    slices = [data._view_rows(x) for x in indices]
+                stat[colname] = [function(x) for x in slices]
         return stat.unselect("_index_", "_group_")
 
     @deco.new_from_generator
