@@ -146,7 +146,7 @@ class ListOfDicts(list):
         of value, it will end up as-is in the output.
 
         >>> from statistics import mean
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.group_by("hood").aggregate(n=len, price=lambda x: mean(x.pluck("price")))
         """
         by = self._group_keys
@@ -173,8 +173,8 @@ class ListOfDicts(list):
         if the correspoding key differs between `self` and `other`.
 
         >>> # All listings that don't have reviews
-        >>> listings = di.ListOfDicts.read_json("data/listings.json")
-        >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
+        >>> listings = di.read_json("data/listings.json")
+        >>> reviews = di.read_json("data/listings-reviews.json")
         >>> listings.anti_join(reviews, "id")
         """
         by1, by2 = self._split_by(*by)
@@ -190,7 +190,7 @@ class ListOfDicts(list):
         """
         Return list with `item` added to the end.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data = data.append(dict.fromkeys(data[0].keys()))
         >>> data.tail()
         """
@@ -202,7 +202,7 @@ class ListOfDicts(list):
         """
         Return list with all items removed.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.clear()
         """
         return self._new([])
@@ -224,7 +224,7 @@ class ListOfDicts(list):
         """
         Return list with items from `other` added to the end.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data = data.extend([dict.fromkeys(data[0].keys())])
         >>> data.tail()
         """
@@ -232,6 +232,28 @@ class ListOfDicts(list):
             other = self.__class__(other)
         yield from itertools.chain(self, other)
 
+    @deco.obsoletes
+    @deco.new_from_generator
+    def fill_missing(self, **key_value_pairs):
+        """
+        Return list with missing keys added.
+
+        If `key_value_pairs` not given, fill all missing keys with ``None``.
+
+        >>> data = di.read_json("data/listings.json")
+        >>> data = data.fill_missing(price=None)
+        >>> data = data.fill_missing()
+        """
+        if not key_value_pairs:
+            keys = util.unique_keys(itertools.chain(*self))
+            key_value_pairs = dict.fromkeys(keys, None)
+        key_value_pairs = key_value_pairs.items()
+        for item in self:
+            for key, value in key_value_pairs:
+                if key not in item:
+                    item[key] = value
+            yield item
+
     @deco.new_from_generator
     def filter(self, function=None, **key_value_pairs):
         """
@@ -242,7 +264,7 @@ class ListOfDicts(list):
         by `key_value_pairs`, which are a shorthand to check against a fixed
         value. See the example below of equivalent filtering with both ways.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.filter(lambda x: x.hood == "Manhattan" and x.guests == 2)
         >>> data.filter(hood="Manhattan", guests=2)
         """
@@ -268,7 +290,7 @@ class ListOfDicts(list):
         by `key_value_pairs`, which are a shorthand to check against a fixed
         value. See the example below of equivalent filtering with both ways.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.filter_out(lambda x: x.hood == "Manhattan")
         >>> data.filter_out(hood="Manhattan")
         """
@@ -285,14 +307,26 @@ class ListOfDicts(list):
                     yield item
 
     @classmethod
-    def from_json(cls, string, **kwargs):
+    def from_json(cls, string, keys=[], types={}, **kwargs):
         """
         Return a new list of dicts from JSON `string`.
+
+        `keys` is an optional list of keys to limit to. `types` is an optional
+        dict mapping keys to datatypes. `kwargs` are passed to ``json.load``.
         """
-        obj = json.loads(string, **kwargs)
-        if not isinstance(obj, list):
+        data = json.loads(string, **kwargs)
+        if not isinstance(data, list):
             raise TypeError("Not a list")
-        return cls(obj)
+        if keys:
+            keys = set(keys)
+            for item in data:
+                for key in set(item) - keys:
+                    del item[key]
+        for key, type in types.items():
+            for item in data:
+                if key in item:
+                    item[key] = type(item[key])
+        return cls(data)
 
     def full_join(self, other, *by):
         """
@@ -305,17 +339,26 @@ class ListOfDicts(list):
         `by` are keys, by which to look for matching items, or tuples of keys
         if the correspoding key differs between `self` and `other`.
 
-        >>> listings = di.ListOfDicts.read_json("data/listings.json")
-        >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
+        >>> listings = di.read_json("data/listings.json")
+        >>> reviews = di.read_json("data/listings-reviews.json")
         >>> listings.full_join(reviews, "id")
         """
-        counter = itertools.count(start=1)
-        other = other.deepcopy().modify(_id_=lambda x: next(counter))
-        # This obsoletes self, @deco.obsoletes not needed.
-        a = self.left_join(other, *by)
-        found_ids = set(x.get("_id_", -1) for x in a)
-        b = other.filter_out(lambda x: x._id_ in found_ids)
-        return (a + b).unselect("_id_")
+        acounter = itertools.count(start=1)
+        bcounter = itertools.count(start=1)
+        a = self.deepcopy().modify(_aid_=lambda x: next(acounter))
+        b = other.deepcopy().modify(_bid_=lambda x: next(bcounter))
+        ab = a.deepcopy().left_join(b, *by)
+        # Fill in missing _bid_ with bogus values.
+        ab = ab.fill_missing(_bid_=next(bcounter))
+        # Check which items of b were not joined into a,
+        # if no items remain, full join is the same as left join ab.
+        b = b.anti_join(ab, "_bid_")
+        if len(b) == 0:
+            return ab.unselect("_aid_", "_bid_")
+        ba = b.left_join(a, *by)
+        # Fill in missing _aid_ with bogus values.
+        ba = ba.fill_missing(_aid_=next(acounter))
+        return (ab + ba).sort(_aid_=1, _bid_=1).unselect("_aid_", "_bid_")
 
     def group_by(self, *keys):
         """
@@ -328,7 +371,7 @@ class ListOfDicts(list):
         """
         Return the first `n` items.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.head(3)
         """
         if n is None:
@@ -348,8 +391,8 @@ class ListOfDicts(list):
         `by` are keys, by which to look for matching items, or tuples of keys
         if the correspoding key differs between `self` and `other`.
 
-        >>> listings = di.ListOfDicts.read_json("data/listings.json")
-        >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
+        >>> listings = di.read_json("data/listings.json")
+        >>> reviews = di.read_json("data/listings-reviews.json")
         >>> listings.inner_join(reviews, "id")
         """
         by1, by2 = self._split_by(*by)
@@ -369,7 +412,7 @@ class ListOfDicts(list):
         """
         Return list with `item` inserted at `index`.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data = data.insert(0, dict.fromkeys(data[0].keys()))
         >>> data.head()
         """
@@ -393,8 +436,8 @@ class ListOfDicts(list):
         `by` are keys, by which to look for matching items, or tuples of keys
         if the correspoding key differs between `self` and `other`.
 
-        >>> listings = di.ListOfDicts.read_json("data/listings.json")
-        >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
+        >>> listings = di.read_json("data/listings.json")
+        >>> reviews = di.read_json("data/listings-reviews.json")
         >>> listings.left_join(reviews, "id")
         """
         by1, by2 = self._split_by(*by)
@@ -415,8 +458,8 @@ class ListOfDicts(list):
         a :class:`ListOfDicts` instance, otherwise the return value will be a
         list of whatever `function` returns.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
-        >>> data.map(lambda x: {**x, "price_per_guest": x.price / x.guests})
+        >>> data = di.read_json("data/listings.json")
+        >>> data.map(lambda x: (x.guests, x.price))
         """
         new = list(map(function, self))
         coerce = all(isinstance(x, dict) for x in new)
@@ -436,7 +479,7 @@ class ListOfDicts(list):
         In `key_function_pairs`, `function` receives as an argument an
         individual item.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.modify(price_per_guest=lambda x: x.price / x.guests)
         """
         key_function_pairs = key_function_pairs.items()
@@ -457,7 +500,7 @@ class ListOfDicts(list):
         In `key_function_pairs`, `function` receives as an argument an
         individual item.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.modify_if(lambda x: x.sqft, price_per_sqft=lambda x: x.price / x.sqft)
         """
         key_function_pairs = key_function_pairs.items()
@@ -479,7 +522,7 @@ class ListOfDicts(list):
 
         `default` is used for items in which `key` is not found.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.pluck("id")[:10]
         """
         return [x.get(key, default) for x in self]
@@ -492,7 +535,7 @@ class ListOfDicts(list):
         but since it's a method, you can use it at the end of a method chain
         instead of wrapping a ``print`` call around the whole chain.
 
-        >>> di.ListOfDicts.read_json("data/listings.json").print_()
+        >>> di.read_json("data/listings.json").print_()
         """
         print(self.to_string(max_items))
 
@@ -503,7 +546,7 @@ class ListOfDicts(list):
         Both keys entirely missing and keys with a value of ``None`` are
         considered missing.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.print_missing_counts()
         """
         print("Missing counts:")
@@ -514,41 +557,54 @@ class ListOfDicts(list):
             print(f"... {key}: {n} ({pc:.1f}%)")
 
     @classmethod
-    def read_csv(cls, fname, encoding="utf_8", header=True, sep=",", columns=None):
+    def read_csv(cls, path, encoding="utf-8", sep=",", header=True, keys=[], types={}):
         """
-        Return a new list from CSV file `fname`.
+        Return a new list from CSV file `path`.
+
+        Will automatically decompress if `path` ends in ``.bz2|.gz|.xz``.
+
+        `keys` is an optional list of keys to limit to. `types` is an optional
+        dict mapping keys to datatypes.
         """
-        with open(fname, "r", encoding=encoding) as f:
+        with util.xopen(path, "rt", encoding=encoding) as f:
             rows = list(csv.reader(f, dialect="unix", delimiter=sep))
             if not rows: return cls([])
-            keys = rows.pop(0) if header else util.generate_colnames(len(rows[0]))
-            if columns:
-                # Limit to requested columns.
-                indices = list(reversed(range(len(rows[0]))))
-                keep = set(keys.index(x) for x in columns)
+            colnames = rows.pop(0) if header else util.generate_colnames(len(rows[0]))
+            if keys:
+                # Drop all keys except the requested ones.
+                drop = [i for i in range(len(rows[0])) if colnames[i] not in keys]
                 for row in rows:
-                    for i in indices:
-                        if i not in keep:
-                            del row[i]
-                keys = columns
-            return cls(dict(zip(keys, x)) for x in rows)
+                    for i in reversed(drop):
+                        del row[i]
+                colnames = keys
+            data = cls(dict(zip(colnames, x)) for x in rows)
+            for key, type in types.items():
+                for item in data:
+                    if key in item:
+                        item[key] = type(item[key])
+            return data
 
     @classmethod
-    def read_json(cls, fname, encoding="utf_8", **kwargs):
+    def read_json(cls, path, encoding="utf-8", keys=[], types={}, **kwargs):
         """
-        Return a new list from JSON file `fname`.
+        Return a new list from JSON file `path`.
 
-        `kwargs` are passed to :meth:`from_json`.
+        Will automatically decompress if `path` ends in ``.bz2|.gz|.xz``.
+
+        `keys` is an optional list of keys to limit to. `types` is an optional
+        dict mapping keys to datatypes. `kwargs` are passed to ``json.load``.
         """
-        with open(fname, "r", encoding=encoding) as f:
-            return cls.from_json(f.read(), **kwargs)
+        with util.xopen(path, "rt", encoding=encoding) as f:
+            return cls.from_json(f.read(), keys, types, **kwargs)
 
     @classmethod
-    def read_pickle(cls, fname):
+    def read_pickle(cls, path):
         """
-        Return a new list from Pickle file `fname`.
+        Return a new list from Pickle file `path`.
+
+        Will automatically decompress if `path` ends in ``.bz2|.gz|.xz``.
         """
-        with open(fname, "rb") as f:
+        with util.xopen(path, "rb") as f:
             return cls(pickle.load(f))
 
     @deco.obsoletes
@@ -557,7 +613,7 @@ class ListOfDicts(list):
         """
         Return items with keys renamed.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.rename(listing_id="id")
         """
         renames = {v: k for k, v in to_from_pairs.items()}
@@ -577,7 +633,7 @@ class ListOfDicts(list):
         """
         Return randomly chosen `n` items.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.sample(3)
         """
         if n is None:
@@ -592,7 +648,7 @@ class ListOfDicts(list):
         """
         Return items, keeping only `keys`.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.select("id", "hood", "zipcode")
         """
         for item in self:
@@ -607,8 +663,8 @@ class ListOfDicts(list):
         if the correspoding key differs between `self` and `other`.
 
         >>> # All listings that have reviews
-        >>> listings = di.ListOfDicts.read_json("data/listings.json")
-        >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
+        >>> listings = di.read_json("data/listings.json")
+        >>> reviews = di.read_json("data/listings-reviews.json")
         >>> listings.semi_join(reviews, "id")
         """
         by1, by2 = self._split_by(*by)
@@ -626,7 +682,7 @@ class ListOfDicts(list):
         `key_dir_pairs` defines the sort order by key with `dir` being ``1``
         for ascending sort, ``-1`` for descending.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.sort(hood=1, zipcode=1)
         """
         data = self
@@ -650,7 +706,7 @@ class ListOfDicts(list):
         """
         Return the last `n` items.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.tail(3)
         """
         if n is None:
@@ -663,9 +719,9 @@ class ListOfDicts(list):
 
     def to_data_frame(self):
         """
-        Return list converted to a :class:`dataiter.DataFrame`.
+        Return list converted to a :class:`.DataFrame`.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.to_data_frame()
         """
         from dataiter import DataFrame
@@ -675,9 +731,9 @@ class ListOfDicts(list):
         """
         Return list converted to a JSON string.
 
-        `kwargs` are passed to ``json.dumps``.
+        `kwargs` are passed to ``json.dump``.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.to_json()[:100]
         """
         kwargs.setdefault("default", str)
@@ -689,7 +745,7 @@ class ListOfDicts(list):
         """
         Return list converted to a ``pandas.DataFrame``.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.to_pandas()
         """
         import pandas as pd
@@ -699,7 +755,7 @@ class ListOfDicts(list):
         """
         Return list as a string formatted for display.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.to_string()
         """
         if max_items is None:
@@ -714,7 +770,7 @@ class ListOfDicts(list):
         """
         Return unique items by `keys`.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.unique("hood")
         """
         if not self: return
@@ -737,7 +793,7 @@ class ListOfDicts(list):
         """
         Return items, dropping `keys`.
 
-        >>> data = di.ListOfDicts.read_json("data/listings.json")
+        >>> data = di.read_json("data/listings.json")
         >>> data.unselect("guests", "sqft", "price")
         """
         for item in self:
@@ -746,44 +802,55 @@ class ListOfDicts(list):
                     del item[key]
             yield item
 
-    def write_csv(self, fname, encoding="utf_8", header=True, sep=","):
+    def write_csv(self, path, encoding="utf-8", header=True, sep=","):
         """
-        Write list to CSV file `fname`.
+        Write list to CSV file `path`.
+
+        Will automatically compress if `path` ends in ``.bz2|.gz|.xz``.
         """
         if not self:
             raise ValueError("Cannot write empty CSV file")
         # Take a superset of all keys.
         keys = util.unique_keys(itertools.chain(*self))
-        util.makedirs_for_file(fname)
-        with open(fname, "w", encoding=encoding) as f:
-            writer = csv.DictWriter(f, keys, dialect="unix", delimiter=sep)
+        util.makedirs_for_file(path)
+        with util.xopen(path, "wt", encoding=encoding) as f:
+            writer = csv.DictWriter(f,
+                                    keys,
+                                    dialect="unix",
+                                    delimiter=sep,
+                                    quoting=csv.QUOTE_MINIMAL)
+
             writer.writeheader() if header else None
             for item in self:
                 # Fill in missing as None.
                 item = {**dict.fromkeys(keys), **item}
                 writer.writerow(item)
 
-    def write_json(self, fname, encoding="utf_8", **kwargs):
+    def write_json(self, path, encoding="utf-8", **kwargs):
         """
-        Write list to JSON file `fname`.
+        Write list to JSON file `path`.
+
+        Will automatically compress if `path` ends in ``.bz2|.gz|.xz``.
 
         `kwargs` are passed to ``json.JSONEncoder``.
         """
         kwargs.setdefault("default", str)
         kwargs.setdefault("ensure_ascii", False)
         kwargs.setdefault("indent", 2)
-        util.makedirs_for_file(fname)
-        with open(fname, "w", encoding=encoding) as f:
+        util.makedirs_for_file(path)
+        with util.xopen(path, "wt", encoding=encoding) as f:
             encoder = json.JSONEncoder(**kwargs)
             for chunk in encoder.iterencode(self):
                 f.write(chunk)
             f.write("\n")
 
-    def write_pickle(self, fname):
+    def write_pickle(self, path):
         """
-        Write list to Pickle file `fname`.
+        Write list to Pickle file `path`.
+
+        Will automatically compress if `path` ends in ``.bz2|.gz|.xz``.
         """
-        util.makedirs_for_file(fname)
-        with open(fname, "wb") as f:
+        util.makedirs_for_file(path)
+        with util.xopen(path, "wb") as f:
             out = [dict(x) for x in self]
             pickle.dump(out, f, pickle.HIGHEST_PROTOCOL)
