@@ -20,10 +20,14 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+import bz2
 import dataiter
+import gzip
 import itertools
+import lzma
 import math
 import numpy as np
+import os
 import shutil
 import string
 
@@ -39,6 +43,13 @@ def count_digits(value):
     m = len(parts[1].rstrip("0"))
     return n, m
 
+def format_alias_doc(alias, target):
+    indent = " " * 8
+    note = (
+        ".. note:: :func:`{}` is a convenience alias for :meth:`{}`."
+        .format(alias.__name__, target.__qualname__))
+    return target.__doc__ + "\n\n" + indent + note
+
 def format_floats(seq):
     precision = dataiter.PRINT_FLOAT_PRECISION
     if any(0 < x < 1/10**precision or x > 10**16 - 1 for x in seq):
@@ -61,8 +72,8 @@ def get_print_width():
 def length(value):
     return 1 if np.isscalar(value) else len(value)
 
-def makedirs_for_file(fname):
-    return Path(fname).parent.mkdir(parents=True, exist_ok=True)
+def makedirs_for_file(path):
+    return Path(path).parent.mkdir(parents=True, exist_ok=True)
 
 @deco.listify
 def pad(strings, align="right"):
@@ -73,6 +84,21 @@ def pad(strings, align="right"):
                if align == "right"
                else value + padding)
 
+def parse_env_boolean(name):
+    return {
+        "1":     True,
+        "t":     True,
+        "true":  True,
+        "y":     True,
+        "yes":   True,
+        "":      False,
+        "0":     False,
+        "f":     False,
+        "false": False,
+        "n":     False,
+        "no":    False,
+    }[os.environ[name].strip().lower()]
+
 def quote(value):
     return '"{}"'.format(str(value).replace('"', r'\"'))
 
@@ -84,6 +110,19 @@ def unique_types(seq):
                x is not None and
                not (isinstance(x, float) and np.isnan(x)))
 
+def xopen(path, mode="r", **kwargs):
+    if "b" not in mode:
+        kwargs.setdefault("encoding", "utf-8")
+    if str(path).endswith(".bz2"):
+        kwargs.setdefault("compresslevel", 6)
+        return bz2.open(path, mode, **kwargs)
+    if str(path).endswith(".gz"):
+        kwargs.setdefault("compresslevel", 6)
+        return gzip.open(path, mode, **kwargs)
+    if str(path).endswith(".xz"):
+        return lzma.open(path, mode)
+    return open(path, mode, **kwargs)
+
 def yield_colnames():
     # Like Excel: a, b, c, ..., aa, bb, cc, ...
     for batch in range(1, 1000):
