@@ -96,6 +96,8 @@ class Vector(np.ndarray):
         >>> vector = di.Vector([0, 1])
         >>> vector.as_boolean()
         """
+        if self.is_string():
+            return self.map(bool)
         return self.astype(bool)
 
     def as_bytes(self):
@@ -168,6 +170,15 @@ class Vector(np.ndarray):
         if self.ndim == 1: return
         raise ValueError(f"Bad dimensions: {self.ndim!r}")
 
+    def drop_missing(self):
+        """
+        Return vector without missing values.
+
+        >>> vector = di.Vector([1, 2, 3, None])
+        >>> vector.drop_missing()
+        """
+        return self[~self.is_missing()].copy()
+
     def equal(self, other):
         """
         Return whether vectors are equal.
@@ -200,7 +211,7 @@ class Vector(np.ndarray):
         values or if you know they are already of the correct type.
         """
         if (hasattr(object, "__iter__") and
-            not isinstance(object, (list, tuple))):
+            not isinstance(object, (np.ndarray, list, tuple))):
             # Evaluate generator/iterator.
             object = list(object)
         return np.array(object, dtype).view(cls)
